/* Ari's JAVA implementation of a labelled, directed edge specifically for the minsum algorithm. */

import java.awt.*;
import java.lang.Math;

public class Edge {
	public Node start;  // the "from" (or start) node
	public Node end;    // the "to" (or end) node
	public String name;   // edge label and characteristics
	private int name_length;
	private int name_height;

	final static private int arrow_height=10;         // the overall height of the arrow
	final static private int arrow_width=13;          // the overall width of the arrow
	final static private int ADJUST_V=5;              // adjust getHeight() so that it doesn't look dorky
	static private double direction_factor;  // angle with which to offset drawing point for directed arrows
	// sets up a new edge from the_start to the_end
	public Edge(Node the_start, Node the_end)

	{
		start=the_start;
		end=the_end;
		name="";
		name_length=0;
		name_height=0;
	}

	public void changeName(String the_name)
		/* %M: name
			 %E: changes the name (label) of the node
			 */
	{
		name=the_name;
		name_length=0; // reset name_length to be recalculated at the next "paint"
		name_height=0;
	}

	// paints the node on the Graphics device
	public void paint(Graphics g) {
		// ASSUMES that nodes are CIRCULAR and OF THE SAME SIZE

		// System.out.println("Drawing edge...");

		// first determine edge endpoints
		int x1, y1;         // the first endpoint
		int x2, y2;         // the first endpoint
		// double slope;       // slope between the first and second nodes
		double angle;       // the angle represented by the slope
		int xdir=MathUtils.sign(end.left-start.left); // xdir=1 if left to right else negative
		int ydir=-MathUtils.sign(end.upper-start.upper); // ydir=1 if down to up else negative
		final int radius=start.width/2; // half the overall width of the circular node

		angle=Math.atan2(start.upper-end.upper,end.left-start.left); // y coordinate order is due to the coordinate system

		// ... marginally correct angle for direction so that directed edges do not coincide
		direction_factor=Math.asin((arrow_height/2)/(double) radius); // so as to make parallel lines of distance arrow_height/2/radius apart
		double correction; // correction to be applied to the angle
		//if (xdir==1) // a stupid type of correction, but it will hopefully work
			correction=direction_factor;
			//else
			//	correction=-direction_factor;

			// System.out.println("Angle is "+(angle*180/Math.PI)+"; correction is "+(correction*180/Math.PI));
		
		x1=(int) (radius*Math.cos(angle+correction));
		x2=(int) (radius*Math.cos(Math.PI+angle-correction));
		// ... compute raw y coordinates (i.e. for the circle)
		y1=(int) (radius*Math.sin(angle+correction));
		y2=(int) (radius*Math.sin(Math.PI+angle-correction));

		// ... compute full x and y coordinates
		x1+=start.left+radius;
		x2+=end.left+radius;
		y1=start.upper+radius-y1;
		y2=end.upper+radius-y2;
		g.drawLine(x1,y1,x2,y2);

		// .... compute arrow
		int centerx=Math.abs((x2+x1)/2);
		int centery=Math.abs((y2+y1)/2);
		Polygon arrow = new Polygon(); // create a new, empty polygon

		final int half_arrow_height=arrow_height/2; // for computational purposes

		int deltax, deltay;            // differentials for drawing the arrow

		// compute differentials
		deltax=(int) (half_arrow_height*Math.cos(Math.PI/2+angle));
		deltay=(int) (half_arrow_height*Math.sin(Math.PI/2+angle));

		arrow.addPoint(centerx+deltax,centery-deltay);
		arrow.addPoint(centerx+(int)(arrow_width*Math.cos(angle)),
									 centery-(int)(arrow_width*Math.sin(angle)));
		arrow.addPoint(centerx-deltax,centery+deltay);
		arrow.addPoint(centerx+deltax,centery-deltay);				

		// draw the arrow
		g.fillPolygon(arrow);

		//name="<"+xdir+","+ydir+">"; // FOR DEBUGGING ONLY!!!
		// compute name parameters
		if (name_length==0) // then recompute it
			name_length=g.getFontMetrics().stringWidth(name);
		if (name_height==0) // then recompute it
			name_height=g.getFontMetrics().getHeight()-ADJUST_V;
		// label the node
		// System.out.println("name_length"+name_length);
		// System.out.println("name_height"+name_height);
		if (ydir>=0 && xdir>=0)         // i.e. text goes above and left of the line
			g.drawString(name,  // make readible
									 centerx+deltax-name_length,
									 centery-deltay-(int) (name_height*Math.sin(Math.PI/2+angle)));
		else if (ydir<=0 && xdir>=0)    // i.e. text goes above and right of the line
			g.drawString(name,  // make readible
									 centerx+deltax,
									 centery-deltay-(int) (name_height*Math.sin(Math.PI/2+angle)));
		else if (ydir>=0 && xdir<=0)    // i.e. text goes below and left of the line
			g.drawString(name,  // make readible
									 centerx+deltax-name_length,
									 centery-deltay-(int) (name_height*Math.sin(Math.PI/2+angle)));
		else                                // i.e. text goes below and right of the line
			g.drawString(name,  // make readible
									 centerx+deltax,
									 centery-deltay-(int) (name_height*Math.sin(Math.PI/2+angle)));

		// g.drawString(name,  // make readible
		// 									 centerx+(int) ((half_arrow_height)*Math.cos(Math.PI/2+angle)),
		// 									 centery-(int) ((half_arrow_height+name_height)*Math.sin(Math.PI/2+angle)));
	}
}
